package com.joinmassive.sampleapp

import android.content.Context
import android.os.Bundle
import android.widget.Button
import android.widget.CheckBox
import android.widget.TextView

import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.isActive
import kotlinx.coroutines.delay

import com.joinmassive.sdk.MassiveClient
import com.joinmassive.sdk.MassiveNotificationOptions
import com.joinmassive.sdk.MassiveOptions
import com.joinmassive.sdk.MassiveServiceType
import kotlinx.coroutines.SupervisorJob

class MainActivity : AppCompatActivity() {
    private lateinit var btnStartStopUsage: Button
    private lateinit var btnRevokeConsent: Button
    private lateinit var tvStatus: TextView
    private lateinit var tvUsage: TextView
    private lateinit var chkAutoStart: CheckBox
    private lateinit var consentHelper: ConsentHelper

    // A coroutine scope to launch the timer for usage tracking.
    private val timerScope = CoroutineScope(Dispatchers.Default + SupervisorJob())

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        // Instantiate a helper class to user's consent for using Massive.
        consentHelper = ConsentHelper(PREFS_NAME, this)

        // Initialize UI controls first.
        initUIControls()
        // Initialize Massive SDK.
        initMassive()
    }

    // Helper method to initialize all UI controls and setup handlers.
    private fun initUIControls() {
        btnStartStopUsage = findViewById(R.id.btnStartStopUsage)
        btnRevokeConsent = findViewById(R.id.btnRevokeConsent)
        tvStatus = findViewById(R.id.tvStatus)
        tvUsage = findViewById(R.id.tvUsage)
        chkAutoStart = findViewById(R.id.chkBoxAutoStart)

        val prefs = getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE)
        chkAutoStart.isChecked = prefs.getBoolean(PREFS_AUTO_START_KEY, true)

        btnStartStopUsage.setOnClickListener {
            when (MassiveClient.state()) {
                MassiveClient.State.Stopped -> {
                    startWithConsent()
                }
                MassiveClient.State.Starting,
                MassiveClient.State.Started -> {
                    stopMassive()
                }
            }
        }

        btnRevokeConsent.setOnClickListener {
            consentHelper.revokeConsent()
            // Stop Massive usage until the user gives consent again.
            stopMassive()
        }

        chkAutoStart.setOnCheckedChangeListener { _, isChecked ->
            with(prefs.edit()) {
                putBoolean(PREFS_AUTO_START_KEY, isChecked)
                apply()
            }
        }
    }

    private fun initMassive() {
        // Check API token to make sure it is set.
        if (!checkToken()) {
            return
        }

        // Initialize Massive SDK with callback to receive the status.
        MassiveClient.init(API_TOKEN, this) {state ->
            updateUI(state)
            // Start monitoring usage after initialization.
            initUsageMonitor()
        }

        // Check if we need to autostart Massive.
        if (chkAutoStart.isChecked) {
            // Use handle start to make sure that we have user's consent.
            startWithConsent()
        }
    }

    private fun initUsageMonitor() {
        timerScope.launch {
            while (isActive) { // Execute until the coroutine is active.
                MassiveClient.usage {
                    it.onSuccess {u ->
                        val trafficKB = u.toDouble() / 1024
                        // It is safe to update UI here because callback is always called in the
                        // main thread.
                        tvUsage.text = getString(R.string.sdk_usage, trafficKB, "KB")
                    }
                }
                // Suspend the coroutine and wait some time before checking usage again.
                delay(USAGE_TRACKING_INTERVAL)
            }
        }
    }

    private fun startWithConsent() {
        // Check if there is user consent to use Massive, and show the dialog if needed.
        if (!consentHelper.isConsentGiven()) {
            consentHelper.showConsentDialog(
                onAccept = {
                    // Start Massive if the user has given consent.
                    startMassive()
                },
                onDecline = {}
            )
        } else {
            startMassive()
        }
    }

    private fun startMassive() {
        // Start massive client with options.
        MassiveClient.start(options = getOptions()) {
            it.result.onSuccess { updateUI(state = MassiveClient.state()) }
            it.result.onFailure { tvStatus.text = getString(R.string.status_usage_start_error) }
        }
    }

    private fun stopMassive() {
        MassiveClient.stop {
            it.result.onSuccess { updateUI(state = MassiveClient.state()) }
            it.result.onFailure { tvStatus.text = getString(R.string.status_usage_stop_error) }
        }
    }

    private fun getOptions(): MassiveOptions {
        return MassiveOptions(
            // Setup options to run Massive as a Foreground service. This mode requires showing a
            // notification. Fill your custom title, content, and icon.
            // For simplicity these options use the icon resource built-in to the SDK library.
            serviceType = MassiveServiceType.Foreground,
            notificationOptions = MassiveNotificationOptions(
                notificationTitle = getString(R.string.notification_title),
                notificationText = getString(R.string.notification_content),
                iconDrawable = com.joinmassive.sdk.R.drawable.massive_sdk_icon
            )
            // Options to initialize Massive as a Background service. Notification is not required.
            // serviceType = MassiveServiceType.Background
        )
    }

    private fun updateUI(state: MassiveClient.State) {
        when (state) {
            MassiveClient.State.Starting -> {
                btnStartStopUsage.isEnabled = true
                tvStatus.text = getString(R.string.status_sdk_initializing)
                btnStartStopUsage.text = getString(R.string.button_text_start_usage)
            }

            MassiveClient.State.Started -> {
                btnStartStopUsage.isEnabled = true
                tvStatus.text = getString(R.string.status_usage_started)
                btnStartStopUsage.text = getString(R.string.button_text_stop_usage)
            }

            MassiveClient.State.Stopped -> {
                btnStartStopUsage.isEnabled = true
                tvStatus.text = getString(R.string.status_usage_stopped)
                btnStartStopUsage.text = getString(R.string.button_text_start_usage)
            }
        }
    }

    private fun checkToken(): Boolean {
        if (API_TOKEN.isNotBlank()) {
            return true
        }

        AlertDialog.Builder(this)
            .setTitle(getString(R.string.token_alert_title))
            .setMessage(getString(R.string.token_alert_msg))
            .create().show()

        return false
    }

    companion object {
        // Set your API token.
        private const val API_TOKEN = ""

        private const val PREFS_NAME = "AppPrefs"
        private const val PREFS_AUTO_START_KEY = "StartMassive"
        private const val USAGE_TRACKING_INTERVAL = 10 * 60 * 1000L // 10 minutes
    }
}
